from inewave.newave.nwv_avl_evap import NwvAvlEvap

from tests.mocks.mock_open import mock_open
from unittest.mock import MagicMock, patch

from tests.mocks.arquivos.nwv_avl_evap import MockNwvAvlEvap


def test_atributos_encontrados_nwv_avl_evap():
    m: MagicMock = mock_open(read_data="".join(MockNwvAvlEvap))
    with patch("builtins.open", m):
        rel = NwvAvlEvap.le_arquivo("")
        assert rel.versao == "FPHA_NEWAVE"
        assert rel.tabela.at[0, "periodo"] == 1
        assert rel.tabela.at[0, "indice_usina"] == 4
        assert rel.tabela.at[0, "nome_usina"] == "FUNIL-GRANDE"
        assert rel.tabela.at[0, "volume_armazenado_hm3"] == 265.86
        assert rel.tabela.at[0, "evaporacao_calculada_hm3"] == 0.23
        assert rel.tabela.at[0, "evaporacao_modelo_hm3"] == 0.23
        assert rel.tabela.at[0, "desvio_absoluto_hm3"] == 0.0
        assert rel.tabela.at[0, "desvio_percentual"] == 0.0


def test_eq_nwv_avl_evap():
    m: MagicMock = mock_open(read_data="".join(MockNwvAvlEvap))
    with patch("builtins.open", m):
        rel1 = NwvAvlEvap.le_arquivo("")
        rel2 = NwvAvlEvap.le_arquivo("")
        assert rel1 == rel2


def test_neq_nwv_avl_evap():
    m: MagicMock = mock_open(read_data="".join(MockNwvAvlEvap))
    with patch("builtins.open", m):
        rel1 = NwvAvlEvap.le_arquivo("")
        rel2 = NwvAvlEvap.le_arquivo("")
        rel1.tabela.iloc[0, 0] = -1
        assert rel1 != rel2



from inewave.newave.modelos.blocos.versaomodelo import VersaoModelo
from inewave.newave.modelos.blocos.tabelacsv import TabelaCSV

from cfinterface.components.block import Block
from cfinterface.files.blockfile import BlockFile
import pandas as pd  # type: ignore
from typing import List, Type, TypeVar, Optional


class ArquivoCSV(BlockFile):
    """
    Modelo de arquivo baseado em blocos específico para o formato
    dos arquivos de saída .CSV do NEWAVE. Espera conter
    a implementação de dois blocos específicos:

    - Versão do modelo
    - Tabela de dados

    O primeiro é genérico, mas o segundo deve ser
    implementado para cada arquivo específico a ser lido.
    """

    BLOCKS: List[Type[Block]] = [VersaoModelo]
    ENCODING = "iso-8859-1"

    T = TypeVar("T")

    def _bloco_por_tipo(self, bloco: Type[T], indice: int) -> Optional[T]:
        """
        Obtém um gerador de blocos de um tipo, se houver algum no arquivo.

        :param bloco: Um tipo de bloco para ser lido
        :type bloco: T
        :param indice: O índice do bloco a ser acessado, dentre os do tipo
        :type indice: int
        :return: O gerador de blocos, se houver
        :rtype: Optional[Generator[T], None, None]
        """
        try:
            return next(
                b
                for i, b in enumerate(self.data.of_type(bloco))
                if i == indice
            )
        except StopIteration:
            return None

    @property
    def versao(self) -> Optional[str]:
        """
        A versão do modelo utilizada para executar o caso.

        :return: A versão do modelo
        :rtype: str | None
        """
        b = self._bloco_por_tipo(VersaoModelo, 0)
        if b is not None:
            return b.data
        return None

    def _tabela(self) -> Optional[pd.DataFrame]:
        """
        A tabela de dados que está contida no arquivo.

        :return: A tabela como um dataframe
        :rtype: pd.DataFrame | None
        """
        b = self._bloco_por_tipo(TabelaCSV, 0)
        if b is not None:
            return b.data
        return None



from inewave.nwlistop.modelos.arquivos.arquivosinpatamar import (
    ArquivoSINPatamar,
)
from inewave.nwlistop.modelos.exces import ExcesAnos


class FteolSIN(ArquivoSINPatamar):
    """
    Armazena os dados das saídas referentes à folga da variável de
    geração eólica para o SIN.

    Esta classe lida com as informações de saída fornecidas pelo
    NWLISTOP e reproduzidas no `fteolsin.out`

    """

    BLOCKS = [
        ExcesAnos,
    ]

    @classmethod
    def le_arquivo(
        cls, diretorio: str, nome_arquivo="fteolsin.out"
    ) -> "FteolSIN":
        return cls.read(diretorio, nome_arquivo)

    def escreve_arquivo(self, diretorio: str, nome_arquivo="fteolsin.out"):
        self.write(diretorio, nome_arquivo)



from cfinterface.files.sectionfile import SectionFile
from inewave.newave.modelos.cortesh import SecaoDadosCortesH
import pandas as pd  # type: ignore


from typing import TypeVar, Optional


class CortesH(SectionFile):
    """
    Armazena os dados de saída do NEWAVE referentes ao
    cabeçalho dos cortes de Benders.
    """

    T = TypeVar("T")

    SECTIONS = [SecaoDadosCortesH]
    STORAGE = "BINARY"

    def __init__(self, data=...) -> None:
        super().__init__(data)
        self.__df: Optional[pd.DataFrame] = None

    @classmethod
    def le_arquivo(
        cls, diretorio: str, nome_arquivo="cortesh.dat"
    ) -> "CortesH":
        return cls.read(diretorio, nome_arquivo)

    def escreve_arquivo(self, diretorio: str, nome_arquivo="cortesh.dat"):
        self.write(diretorio, nome_arquivo)

    def cabecalho(self):
        registro_cabecalho = [r for r in self.data.of_type(SecaoDadosCortesH)]
        if len(registro_cabecalho) > 0:
            return registro_cabecalho
        else:
            return None

    # def __monta_df_de_registros(self) -> Optional[pd.DataFrame]:
    #     registros: List[RegistroUHEHidr] = [
    #         r for r in self.data.of_type(RegistroUHEHidr)
    #     ]
    #     if len(registros) == 0:
    #         return None
    #     df = pd.DataFrame(
    #         columns=[
    #             "nome_usina",
    #             "posto",
    #             "submercado",
    #             "empresa",
    #             "codigo_usina_jusante",
    #             "desvio",
    #             "volume_minimo",
    #             "volume_maximo",
    #             "volume_vertedouro",
    #             "volume_desvio",
    #             "cota_minima",
    #             "cota_maxima",
    #             *[f"a{i}_volume_cota" for i in range(5)],
    #             *[f"a{i}_cota_area" for i in range(5)],
    #             *[f"evaporacao_{m}" for m in MESES_ABREV],
    #             "numero_conjuntos_maquinas",
    #             *[f"maquinas_conjunto_{i}" for i in range(1, 6)],
    #             *[f"potencia_nominal_conjunto_{i}" for i in range(1, 6)],
    #             *[f"queda_nominal_conjunto_{i}" for i in range(1, 6)],
    #             *[f"vazao_nominal_conjunto_{i}" for i in range(1, 6)],
    #             "produtibilidade_especifica",
    #             "perdas",
    #             "numero_polinomios_jusante",
    #             *[f"a{i}_jusante_1" for i in range(5)],
    #             *[f"a{i}_jusante_2" for i in range(5)],
    #             *[f"a{i}_jusante_3" for i in range(5)],
    #             *[f"a{i}_jusante_4" for i in range(5)],
    #             *[f"a{i}_jusante_5" for i in range(5)],
    #             *[f"a{i}_jusante_6" for i in range(5)],
    #             *[f"referencia_jusante_{i}" for i in range(1, 7)],
    #             "canal_fuga_medio",
    #             "influencia_vertimento_canal_fuga",
    #             "fator_carga_maximo",
    #             "fator_carga_minimo",
    #             "vazao_minima_historica",
    #             "numero_unidades_base",
    #             "tipo_turbina",
    #             "representacao_conjunto",
    #             "teif",
    #             "ip",
    #             "tipo_perda",
    #             "data",
    #             "observacao",
    #             "volume_referencia",
    #             "tipo_regulacao",
    #         ]
    #     )
    #     for i, r in enumerate(registros):
    #         df.loc[i + 1] = [
    #             r.nome,
    #             r.posto,
    #             r.subsistema,
    #             r.empresa,
    #             r.jusante,
    #             r.desvio,
    #             r.volume_minimo,
    #             r.volume_maximo,
    #             r.volume_vertedouro,
    #             r.volume_desvio,
    #             r.cota_minima,
    #             r.cota_maxima,
    #             *(r.polinomio_volume_cota),
    #             *(r.polinomio_cota_area),
    #             *(r.evaporacao),
    #             r.numero_conjuntos_maquinas,
    #             *(r.numero_maquinas_conjunto),
    #             *(r.potef_conjunto),
    #             *(r.hef_conjunto),
    #             *(r.qef_conjunto),
    #             r.produtibilidade_especifica,
    #             r.perdas,
    #             r.numero_polinomios_jusante,
    #             *(r.polinomios_jusante),
    #             r.canal_fuga_medio,
    #             r.influencia_vertimento_canal_fuga,
    #             r.fator_carga_maximo,
    #             r.fator_carga_minimo,
    #             r.vazao_minima_historica,
    #             r.numero_unidades_base,
    #             r.tipo_turbina,
    #             r.representacao_conjunto,
    #             r.teif,
    #             r.ip,
    #             r.tipo_perda,
    #             r.data_referencia,
    #             r.observacao,
    #             r.volume_referencia,
    #             r.tipo_regulacao,
    #         ]
    #     df.index.name = "codigo_usina"

    #     df = df.astype(
    #         {
    #             "nome_usina": str,
    #             "posto": int,
    #             "submercado": int,
    #             "empresa": int,
    #             "codigo_usina_jusante": int,
    #             "desvio": int,
    #             "volume_minimo": float,
    #             "volume_maximo": float,
    #             "volume_vertedouro": float,
    #             "volume_desvio": float,
    #             "cota_minima": float,
    #             "cota_maxima": float,
    #             "produtibilidade_especifica": float,
    #             "perdas": float,
    #             "canal_fuga_medio": float,
    #             "influencia_vertimento_canal_fuga": int,
    #             "fator_carga_maximo": float,
    #             "fator_carga_minimo": float,
    #             "vazao_minima_historica": int,
    #             "numero_unidades_base": int,
    #             "tipo_turbina": int,
    #             "representacao_conjunto": int,
    #             "teif": float,
    #             "ip": float,
    #             "tipo_perda": int,
    #             "data": str,
    #             "observacao": str,
    #             "volume_referencia": float,
    #             "tipo_regulacao": str,
    #         },
    #     )
    #     return df

    # def __atualiza_registros(self):
    #     registros: List[RegistroUHEHidr] = [r for r in self.data][1:]
    #     for (_, linha), r in zip(self.cadastro.iterrows(), registros):
    #         r.nome = linha["nome_usina"]
    #         r.posto = linha["posto"]
    #         r.subsistema = linha["submercado"]
    #         r.empresa = linha["empresa"]
    #         r.jusante = linha["codigo_usina_jusante"]
    #         r.desvio = linha["desvio"]
    #         r.volume_minimo = linha["volume_minimo"]
    #         r.volume_maximo = linha["volume_maximo"]
    #         r.volume_vertedouro = linha["volume_vertedouro"]
    #         r.volume_desvio = linha["volume_desvio"]
    #         r.cota_minima = linha["cota_minima"]
    #         r.cota_maxima = linha["cota_maxima"]
    #         r.polinomio_volume_cota = linha[
    #             [f"a{i}_volume_cota" for i in range(5)]
    #         ].tolist()
    #         r.polinomio_cota_area = linha[
    #             [f"a{i}_cota_area" for i in range(5)]
    #         ].tolist()
    #         r.evaporacao = linha[
    #             [f"evaporacao_{m}" for m in MESES_ABREV]
    #         ].tolist()
    #         r.numero_conjuntos_maquinas = linha["numero_conjuntos_maquinas"]
    #         r.potef_conjunto = linha[
    #             [f"potencia_nominal_conjunto_{i}" for i in range(1, 6)]
    #         ].tolist()
    #         r.hef_conjunto = linha[
    #             [f"queda_nominal_conjunto_{i}" for i in range(1, 6)]
    #         ].tolist()
    #         r.qef_conjunto = linha[
    #             [f"vazao_nominal_conjunto_{i}" for i in range(1, 6)]
    #         ].tolist()
    #         r.produtibilidade_especifica = linha["produtibilidade_especifica"]
    #         r.perdas = linha["perdas"]
    #         r.numero_polinomios_jusante = linha["numero_polinomios_jusante"]
    #         r.polinomios_jusante = linha[
    #             [f"a{i}_jusante_1" for i in range(5)]
    #             + [f"a{i}_jusante_2" for i in range(5)]
    #             + [f"a{i}_jusante_3" for i in range(5)]
    #             + [f"a{i}_jusante_4" for i in range(5)]
    #             + [f"a{i}_jusante_5" for i in range(5)]
    #             + [f"a{i}_jusante_6" for i in range(5)]
    #             + [f"referencia_jusante_{i}" for i in range(1, 7)]
    #         ].tolist()
    #         r.canal_fuga_medio = linha["canal_fuga_medio"]
    #         r.influencia_vertimento_canal_fuga = linha[
    #             "influencia_vertimento_canal_fuga"
    #         ]
    #         r.fator_carga_maximo = linha["fator_carga_maximo"]
    #         r.fator_carga_minimo = linha["fator_carga_minimo"]
    #         r.vazao_minima_historica = linha["vazao_minima_historica"]
    #         r.numero_unidades_base = linha["numero_unidades_base"]
    #         r.tipo_turbina = linha["tipo_turbina"]
    #         r.representacao_conjunto = linha["representacao_conjunto"]
    #         r.teif = linha["teif"]
    #         r.ip = linha["ip"]
    #         r.tipo_perda = linha["tipo_perda"]
    #         r.data_referencia = linha["data"]
    #         r.observacao = linha["observacao"]
    #         r.volume_referencia = linha["volume_referencia"]
    #         r.tipo_regulacao = linha["tipo_regulacao"]

    # @property
    # def cadastro(self) -> pd.DataFrame:
    #     """
    #     Obtém a tabela com os dados cadastrais existentes no arquivo
    #     binário.

    #     - nome_usina (`str`): nome da usina (12 caracteres)
    #     - posto (`int`): posto de vazão natural da usina
    #     - submercado (`int`): submercado da usina
    #     - empresa (`int`): agente responsável pela usina
    #     - codigo_usina_jusante (`int`): posto à jusante da usina
    #     - desvio (`float`): TODO
    #     - volume_minimo (`float`): volume mínimo da usina (hm3)
    #     - volume_maximo (`float`): volume máximo da usina (hm3)
    #     - volume_vertedouro (`float`): volume do vertedouro da usina (hm3)
    #     - volume_desvio (`float`): TODO
    #     - cota_minima (`float`): cota mínima da usina (m)
    #     - cota_maxima (`float`): cota máxima da usina (m)
    #     - a[0-4]_volume_cota (`float`): coeficientes do polinômio volume-cota
    #     - a[0-4]_cota_area (`float`): coeficientes do polinômio cota-área
    #     - evaporacao_[JAN..DEZ] (`float`): coeficientes de evaporação (mm)
    #     - numero_conjuntos_maquinas (`int`): número de conjuntos de máquinas
    #     - maquinas_conjunto_[1-5] (`int`): máquinas por conjunto
    #     - potencia_nominal_conjunto_[1-5] (`float`): potência das máquinas (MWmed)
    #     - queda_nominal_conjunto_[1-5]: alturas nominais de queda por conjunto (m)
    #     - vazao_nominal_conjunto_[1-5]: vazões nominais por conjunto (m3/s)
    #     - produtibilidade_especifica (`float`): produtibilidade específica
    #     - perdas (`float`): perdas da usina
    #     - numero_polinomios_jusante (`int`): número de polinômios de jusante
    #     - a[0-4]_jusante_[1-6] (`float`): coeficientes de cada polinjus
    #     - referencia_jusante_[1-6] (`float`): coeficientes do polinjus de referência
    #     - canal_fuga_medio (`float`): cota média do canal de fuga (m)
    #     - influencia_vertimento_canal_fuga (`int`): TODO (0 ou 1)
    #     - fator_carga_maximo (`float`): TODO (%)
    #     - fator_carga_minimo (`float`): TODO (%)
    #     - vazao_minima_historica (`float`): vazão mínima da usina (m3/s)
    #     - numero_unidades_base (`int`): TODO
    #     - tipo_turbina (`int`): TODO
    #     - representacao_conjunto (`int`): TODO
    #     - teif (`float`): TODO (%)
    #     - ip (`float`): TODO (%)
    #     - tipo_perda (`int`): TODO
    #     - data (`str`): DD-MM-AA
    #     - observacao (`str`): observação qualquer sobre a usina
    #     - volume_referencia (`float`): TODO (hm3)
    #     - tipo_regulacao (`str`): D, S ou M

    #     :return: A tabela com os dados cadastrais
    #     :rtype: List[pd.DataFrame]
    #     """
    #     if self.__df is None:
    #         self.__df = self.__monta_df_de_registros()
    #     return self.__df

    # @cadastro.setter
    # def cadastro(self, df: pd.DataFrame):
    #     self.__df = df



from cfinterface.components.section import Section
from cfinterface.components.line import Line
from cfinterface.components.integerfield import IntegerField
from typing import IO


class SecaoDadosCortesH(Section):
    """
    Registro com os dados da execução do caso existente no
    arquivo cortesh.dat
    """

    def __init__(self, previous=None, next=None, data=None) -> None:
        super().__init__(previous, next, data)
        self.__linha_cabecalho = Line(
            [
                IntegerField(size=4, starting_position=0),  # Versão NEWAVE
                IntegerField(size=4, starting_position=4),  # Tamanho Cortes
                IntegerField(size=4, starting_position=8),  # Tamanho Estados
                IntegerField(size=4, starting_position=12),  # Num REEs
                IntegerField(size=4, starting_position=16),  # Num Períodos PRE
                IntegerField(size=4, starting_position=20),  # Num Períodos EST
                IntegerField(size=4, starting_position=24),  # Num Períodos POS
                IntegerField(
                    size=4, starting_position=28
                ),  # Num Configurações
                IntegerField(size=4, starting_position=32),  # NPEA - ??
                IntegerField(size=4, starting_position=36),  # Num Forwards
                IntegerField(size=4, starting_position=40),  # Num Pat. Carga
                IntegerField(size=4, starting_position=44),  # Ano Inic. EST
                IntegerField(size=4, starting_position=48),  # Mes Inic. EST
                IntegerField(size=4, starting_position=52),  # Lag Máximo GNL
                IntegerField(size=4, starting_position=56),  # Mec. Aversão
                IntegerField(size=4, starting_position=60),  # Num Submercados
                IntegerField(size=4, starting_position=64),  # Num Subm + Fict
            ],
            storage="BINARY",
        )

    def __eq__(self, o: object) -> bool:
        if not isinstance(o, SecaoDadosCortesH):
            return False
        bloco: SecaoDadosCortesH = o
        if not all(
            [
                isinstance(self.data, list),
                isinstance(o.data, list),
            ]
        ):
            return False
        else:
            return self.data == bloco.data

    def read(self, file: IO, *args, **kwargs):
        dados_cabecalho = self.__linha_cabecalho.read(
            file.read(self.__linha_cabecalho.size)
        )
        self.__numero_estagios = sum(dados_cabecalho[4:7])
        indices_ultimos_cortes_periodo = Line(
            [
                IntegerField(size=1, starting_position=0 + 1 * i)
                for i in range(self.__numero_estagios)
            ],
            storage="BINARY",
        )
        dados_ultimos_cortes = indices_ultimos_cortes_periodo.read(
            file.read(indices_ultimos_cortes_periodo.size)
        )
        self.data = dados_cabecalho + dados_ultimos_cortes

    @property
    def versao_newave(self) -> int:
        return self.data[0]

    @versao_newave.setter
    def versao_newave(self, v: int):
        self.data[0] = v

    @property
    def tamanho_corte(self) -> int:
        return self.data[1]

    @tamanho_corte.setter
    def tamanho_corte(self, v: int):
        self.data[1] = v

    @property
    def tamanho_estado(self) -> int:
        return self.data[2]

    @tamanho_estado.setter
    def tamanho_estado(self, v: int):
        self.data[2] = v

    @property
    def numero_rees(self) -> int:
        return self.data[3]

    @numero_rees.setter
    def numero_rees(self, v: int):
        self.data[3] = v

    @property
    def numero_periodos_pre(self) -> int:
        return self.data[4]

    @numero_periodos_pre.setter
    def numero_periodos_pre(self, v: int):
        self.data[4] = v

    @property
    def numero_periodos_estudo(self) -> int:
        return self.data[5]

    @numero_periodos_estudo.setter
    def numero_periodos_estudo(self, v: int):
        self.data[5] = v

    @property
    def numero_periodos_pos(self) -> int:
        return self.data[6]

    @numero_periodos_pos.setter
    def numero_periodos_pos(self, v: int):
        self.data[6] = v

    @property
    def numero_configuracoes(self) -> int:
        return self.data[7]

    @numero_configuracoes.setter
    def numero_configuracoes(self, v: int):
        self.data[7] = v

    @property
    def numero_forwards(self) -> int:
        return self.data[9]

    @numero_forwards.setter
    def numero_forwards(self, v: int):
        self.data[9] = v

    @property
    def numero_patamares(self) -> int:
        return self.data[10]

    @numero_patamares.setter
    def numero_patamares(self, v: int):
        self.data[10] = v

    @property
    def ano_inicio_estudo(self) -> int:
        return self.data[11]

    @ano_inicio_estudo.setter
    def ano_inicio_estudo(self, v: int):
        self.data[11] = v

    @property
    def mes_inicio_estudo(self) -> int:
        return self.data[12]

    @mes_inicio_estudo.setter
    def mes_inicio_estudo(self, v: int):
        self.data[12] = v

    @property
    def lag_maximo_gnl(self) -> int:
        return self.data[13]

    @lag_maximo_gnl.setter
    def lag_maximo_gnl(self, v: int):
        self.data[13] = v

    @property
    def mecanismo_aversao(self) -> int:
        return self.data[14]

    @mecanismo_aversao.setter
    def mecanismo_aversao(self, v: int):
        self.data[14] = v

    @property
    def numero_submercados(self) -> int:
        return self.data[15]

    @numero_submercados.setter
    def numero_submercados(self, v: int):
        self.data[15] = v

    @property
    def numero_total_submercados(self) -> int:
        return self.data[16]

    @numero_total_submercados.setter
    def numero_total_submercados(self, v: int):
        self.data[16] = v



from inewave.newave.modelos.enavazf import SecaoDadosEnavazf
from inewave.newave.enavazf import Enavazf

from os.path import join
from tests.mocks.mock_open import mock_open
from unittest.mock import MagicMock, patch


ARQ_TEST = "./tests/mocks/arquivos"

NUM_FORWARDS = 2
NUM_REES = 1
NUM_ESTAGIOS = 16
NUM_ESTAGIOS_TH = 12
NUM_ENTRADAS = NUM_FORWARDS * NUM_REES * (NUM_ESTAGIOS_TH + NUM_ESTAGIOS)


def test_secao_enavaz():
    r = SecaoDadosEnavazf()
    with open(join(ARQ_TEST, "enavazf.dat"), "rb") as fp:
        r.read(
            fp,
            numero_forwards=NUM_FORWARDS,
            numero_rees=NUM_REES,
            numero_estagios=NUM_ESTAGIOS,
            numero_estagios_th=NUM_ESTAGIOS_TH,
        )

    assert len(r.data) == NUM_ENTRADAS


def test_atributos_encontrados_enavaz():
    h = Enavazf.le_arquivo(
        ARQ_TEST,
        numero_forwards=NUM_FORWARDS,
        numero_rees=NUM_REES,
        numero_estagios=NUM_ESTAGIOS,
        numero_estagios_th=NUM_ESTAGIOS_TH,
    )
    assert h.series is not None
    assert h.series.isna().sum().sum() == 0


def test_atributos_nao_encontrados_enavaz():
    m: MagicMock = mock_open(read_data="")
    with patch("builtins.open", m):
        h = Enavazf.le_arquivo(
            "",
            numero_forwards=NUM_FORWARDS,
            numero_rees=NUM_REES,
            numero_estagios=NUM_ESTAGIOS,
            numero_estagios_th=NUM_ESTAGIOS_TH,
        )
        assert h.series.isna().sum().sum() == NUM_ENTRADAS


def test_eq_enavaz():
    h1 = Enavazf.le_arquivo(
        ARQ_TEST,
        numero_forwards=NUM_FORWARDS,
        numero_rees=NUM_REES,
        numero_estagios=NUM_ESTAGIOS,
        numero_estagios_th=NUM_ESTAGIOS_TH,
    )
    h2 = Enavazf.le_arquivo(
        ARQ_TEST,
        numero_forwards=NUM_FORWARDS,
        numero_rees=NUM_REES,
        numero_estagios=NUM_ESTAGIOS,
        numero_estagios_th=NUM_ESTAGIOS_TH,
    )
    assert h1 == h2



from cfinterface.files.sectionfile import SectionFile
from inewave.newave.modelos.vazaos import SecaoDadosVazaos
import pandas as pd  # type: ignore

from typing import TypeVar, Optional


class Vazaos(SectionFile):
    """
    Armazena os dados de saída do NEWAVE referentes às séries sintéticas
    de vazão para a simulação final geradas pelo modelo.
    """

    T = TypeVar("T")

    SECTIONS = [SecaoDadosVazaos]
    STORAGE = "BINARY"

    def __init__(self, data=...) -> None:
        super().__init__(data)

    @classmethod
    def le_arquivo(
        cls,
        diretorio: str,
        nome_arquivo="vazaos.dat",
        numero_series: int = 2000,
        numero_uhes: int = 164,
        numero_estagios: int = 60,
        numero_estagios_th: int = 12,
    ) -> "Vazaos":
        return cls.read(
            diretorio,
            nome_arquivo,
            numero_series=numero_series,
            numero_uhes=numero_uhes,
            numero_estagios=numero_estagios,
            numero_estagios_th=numero_estagios_th,
        )

    def escreve_arquivo(self, diretorio: str, nome_arquivo="vazaos.dat"):
        self.write(diretorio, nome_arquivo)

    @property
    def series(self) -> Optional[pd.DataFrame]:
        """
        Obtém a tabela com os dados das séries de vazão
        afluente por UHE e por estágio.

        - estagio (`int`): estágio do cenário gerado
        - uhe (`int`): UHE para a qual foi gerado
        - serie (`int`): índice da série sintética
        - valor (`float`): vazão em hm3

        :return: A tabela com os dados das séries
        :rtype: pd.DataFrame | None
        """
        sections = [r for r in self.data.of_type(SecaoDadosVazaos)]
        if len(sections) > 0:
            return sections[0].data
        else:
            return None

    @series.setter
    def series(self, df: pd.DataFrame):
        sections = [r for r in self.data.of_type(SecaoDadosVazaos)]
        if len(sections) > 0:
            sections[0].data = df



from typing import Optional
from cfinterface.components.register import Register
from cfinterface.components.line import Line
from cfinterface.components.integerfield import IntegerField
from cfinterface.components.datetimefield import DatetimeField
from cfinterface.components.floatfield import FloatField
from datetime import datetime


class RegistroEolicaFTE(Register):
    """
    Registro que contém as informações da função de produção
    linear vento-potência para as usinas eólicas.
    """

    IDENTIFIER = "EOLICA-FUNCAO-PRODUCAO-VENTO-POTENCIA-LINEAR-PU-PERIODO"
    IDENTIFIER_DIGITS = 55
    LINE = Line(
        [
            IntegerField(),
            DatetimeField(size=7, format="%Y/%m"),
            DatetimeField(size=7, format="%Y/%m"),
            FloatField(decimal_digits=8),
            FloatField(decimal_digits=8),
        ],
        delimiter=";",
    )

    @property
    def codigo_eolica(self) -> Optional[int]:
        return self.data[0]

    @codigo_eolica.setter
    def codigo_eolica(self, c: int):
        self.data[0] = c

    @property
    def data_inicial(self) -> Optional[datetime]:
        return self.data[1]

    @data_inicial.setter
    def data_inicial(self, c: datetime):
        self.data[1] = c

    @property
    def data_final(self) -> Optional[datetime]:
        return self.data[2]

    @data_final.setter
    def data_final(self, c: datetime):
        self.data[2] = c

    @property
    def coeficiente_linear(self) -> Optional[float]:
        return self.data[3]

    @coeficiente_linear.setter
    def coeficiente_linear(self, c: float):
        self.data[3] = c

    @property
    def coeficiente_angular(self) -> Optional[float]:
        return self.data[4]

    @coeficiente_angular.setter
    def coeficiente_angular(self, c: float):
        self.data[4] = c


class RegistroPEEFTE(Register):
    """
    Registro que contém as informações da função de produção
    linear vento-potência para um PEE.
    """

    IDENTIFIER = "PEE-FPVP-LIN-PU-PER"
    IDENTIFIER_DIGITS = 19
    LINE = Line(
        [
            IntegerField(),
            DatetimeField(size=7, format="%Y/%m"),
            DatetimeField(size=7, format="%Y/%m"),
            FloatField(size=20, decimal_digits=17),
            FloatField(size=20, decimal_digits=17),
        ],
        delimiter=";",
    )

    @property
    def codigo_pee(self) -> Optional[int]:
        return self.data[0]

    @codigo_pee.setter
    def codigo_pee(self, c: int):
        self.data[0] = c

    @property
    def data_inicial(self) -> Optional[datetime]:
        return self.data[1]

    @data_inicial.setter
    def data_inicial(self, c: datetime):
        self.data[1] = c

    @property
    def data_final(self) -> Optional[datetime]:
        return self.data[2]

    @data_final.setter
    def data_final(self, c: datetime):
        self.data[2] = c

    @property
    def coeficiente_linear(self) -> Optional[float]:
        return self.data[3]

    @coeficiente_linear.setter
    def coeficiente_linear(self, c: float):
        self.data[3] = c

    @property
    def coeficiente_angular(self) -> Optional[float]:
        return self.data[4]

    @coeficiente_angular.setter
    def coeficiente_angular(self, c: float):
        self.data[4] = c
